import Promise from 'bluebird'
import util from 'util'
import p from 'path'
import {EOL as eol} from 'os'
import schema from './userSchema.js'
import mkdirp from 'mkdirp'

let debug = require('debug')('explorer:users')
let fs = Promise.promisifyAll(require('fs'))
let bcrypt = Promise.promisifyAll(require('bcrypt'))

const  saltLength = 9
const  keyLength = 25

/**
 * Database handler
 */
class Users {

  /**
   * @constructor
   * @param object options (database path)
   * @return void
   */
  constructor(options) {
    options = options || {} 

    this.database = options.database || p.resolve(__dirname, '../data/users')
  }

  /**
   * Load database from file
   * @return Promise
   */
  load() {
    let self = this

    return this.parse()  
    .then(function(users) {
      debug('DB Loaded, %o', users)
      self.users = users 
      return Promise.resolve()
    })
    .catch(function(e) {
      console.error('Can not load database')
      return Promise.reject(e)
    })
  }

  /**
   * Remove a user from the file
   * @param string username
   * @return Promise
   */
  remove(username) {
    let u = this.users.findIndex(u => u.username == username)

    if(!~u) {
      return Promise.reject(`User ${username} not found`)
    }

    this.users.splice(u, 1)

    return this.write()
  }

  /**
   * Put a user to the file
   * @param string username
   * @return Promise
   */
  put(user) {
    if(!(user instanceof User))
      throw new TypeError('Not a User instance')

    let u = this.users.findIndex(e => e.username == user.username)

    if(~u) {
      debug('update user %s, %s', user.username, user)
      this.users[u] = user
    } else {
      debug('user created %s', user)
      this.users.push(user) 
    }

    return this.write()
  }

  /**
   * Writes the database
   * @return Promise
   */
  write() {
    let str = []

    for(let i of this.users) {
      i && str.push(i.toString())
    }

    return fs.writeFileAsync(this.database, str.join(eol))
  }

  /**
   * Parse the database
   * @return Promise
   */
  parse() {
   return fs.readFileAsync(this.database, {encoding: 'utf-8'}) 
   .then(function(db) {
     
     db = db.split(eol).filter(v => v.length > 0)

     let promises = db.map(e => e.split(':'))
     .map(function(e) {
        let u = {};
        for(let i in schema) {
          u[i] = e[schema[i].position] 

          if(u[i] === undefined)
            u[i] = schema[i].default 

          if(schema[i].type == 'boolean') {
            u[i] = u[i] === '1' ? true : false 
          } else if (schema[i].type == 'buffer') {
            u[i] = new Buffer(u[i], 'base64').toString('ascii')
          }
        } 
        
        return new User(u, false)
     })

     return Promise.all(promises)
   })
  }

  /**
   * Gets a User by username
   * @param string username
   * @return User
   */
  get(username) {
    return this.users.find(u => u.username == username)
  }

  /**
   * Gets a User by key
   * @return User
   */
  getByKey(key) {
    return this.users.find(u => u.key == key)
  }

  /**
   * Deletes a user by username
   * @param string username
   * @return Promise
   */
  delete(username) {
    let u = this.users.findIndex(e => e.username == username)

    if(!~u) {
      return Promise.reject(`User ${username} does not exist`) 
    }

    this.users.splice(u, 1)

    return this.write()
  }

  /**
   * Authenticates a user
   * @param string username
   * @param string password
   * @return Promise
   */
  authenticate(username, password) {
    let u = this.get(username) 

    if(!u) {
      debug('User not found')
      return Promise.reject(`No user with the username ${username}`)
    }

    return bcrypt.compareAsync(password, u.password)
    .then(ok => Promise.resolve(ok))
    .catch(function(err) {
      console.error(err) 
      return Promise.reject(`Wrong password`)
    })
  }
}

/**
 * User class
 */
class User {

  /**
   * @constructor
   * @param string user
   * @param boolean crypt wheter to encrypt password or not
   * @return Promise
   */
  constructor(user, crypt = true) {

    user = this.sanitize(user)

    debug('New User %o', user)

    if(!this.isValid(user)) {
      throw new TypeError("User is not valid") 
    }

    for(var i in user) {
      if(!this[i])
        this[i] = user[i]
    }

    if(!crypt)
      return Promise.resolve(this)

    return this.crypt()
  }

  /**
   * getCookie
   * @return object safe user object (remove password,salt)
   */
  getCookie() {
    return {
      username: this.username, 
      home: this.home, 
      key: this.key
    } 
  }

  /**
   * sanitize a user object according to the schema
   * @param object user
   * @return object sanitized user
   */
  sanitize(user) {
    for(let i in schema) {
      if(typeof user[i] === 'undefined' && schema[i].default !== undefined) {
        user[i] = schema[i].default 
      } else if(schema[i].type == 'boolean') {
        user[i] = this.valueToIntegerBool(user[i])
      } else if(schema[i].type == 'buffer' && user[i].length && typeof user[i] == 'string') {
        user[i] = user[i].split(eol).filter(v => v.length > 0)
      }
    }

    return user
  }

  /**
   * Valids a user through the schema
   * @param user
   * @return boolean
   */
  isValid(user) {
    let valid = Object.keys(schema).filter(e => !user.hasOwnProperty(e) || /:/g.test(user[e]))

    return valid.length == 0
  }

  /**
   * valueToIntegerBool 
   * transforms :
   *   - '1', 1, true to 1 
   *   - '0', 0, false to 0 
   * @throws TypeError
   * @param mixed v
   * @return integer
   */
  valueToIntegerBool(v) {
    if(typeof v == 'boolean') {
      return v === true ? 1 : 0
    }

    if(v !== undefined) {
      //json outputs an integer, body outputs a string
      return v = parseInt(''+v) === 1 ? 1 : 0
    }

    return 0
  }

  /**
   * Crypt 
   * Hash this.password
   * @return Promise
   */
  crypt() {
    let self = this
    return bcrypt.hashAsync(this.password, saltLength)
   .then(function(hash) {
      self.password = hash; 
      return Promise.resolve(self)
   })
   .catch(function(err) {
      console.error(err) 
      return Promise.reject('Could not hash password')
   })
  }

  /**
   * Generates the alphanumerical user key
   * @return Promise
   */
  generateKey() {
    let self = this

    //using bcrypt to generate a key
    return bcrypt.genSaltAsync(saltLength)
    .then(function(salt) {
      return bcrypt.hashAsync(salt, saltLength)
    })
    .then(function(hash) {
      let l = hash.length 
      let s = []
      let i = 0
      //take only alpha-numeric chars
      while(l-- && i < keyLength ) {
        let c = hash.charAt(l)
        if(/[a-z0-9]/i.test(c)) {
          s.push(c) 
          i++
        }
      }

      self.key = s.join('')
      return Promise.resolve(self)
    })
    .catch(function(err) {
      console.error(err) 
      return Promise.reject('Could not generate key')
    })
  }

  /**
   * Updates the current user with new values
   * @param object user the new user object
   * @param array ignore keys to be ignored
   * @return Promise
   */
  update(user, ignore) {

    ignore = ignore ? ignore : []

    for(let i in schema) {
      if(schema[i].update === false && !~ignore.indexOf(i))  {
        ignore.push(i)
      }
    }

    let crypt = true

    //we don't want to update password
    if(user.password == '' || typeof user.password == 'undefined') {
      crypt = false
    }

    user = this.sanitize(user)

    for(let i in user) {
      if(~ignore.indexOf(i) || !~Object.keys(schema).indexOf(i) || typeof user[i] === 'undefined')
        continue;

      if(i == 'password' && crypt === false)
        continue;

      if(schema[i].directory === true && user[i].length) {
        let dir = p.resolve(user.home, user[i])

        if(!fs.existsSync(dir)) {
          mkdirp.sync(dir) 
        }
      }

      this[i] = user[i]
    }

    //we want to update the key (form boolean checkbox/option)
    if(''+user.key === '1') {
      return this.generateKey()
      .then(u => crypt ? u.crypt() : Promise.resolve(u))
    }

    return crypt ? this.crypt() : Promise.resolve(this)
  }

  /**
   * Returns the user string for database
   * @return string
   */
  toString() {
    let str = []

    for(let p in schema) {
      if(schema[p].type == 'buffer') {
        var b = util.isArray(this[p]) ? this[p].join(eol) : this[p]
        str.push(new Buffer(b).toString('base64'))
      } else {
        str.push(`${this[p]}`)
      }
    }

    return str.join(':')
  }
}

export {User, Users}
